@@ -27,6 +27,20 @@
 # ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
+
+"""
+
+`typepadapp.views.base` provides a basis for building Django views that use
+TypePad API resources.
+
+As TypePad API resources are best requested in a single batch request, the
+class-based view implementation in `TypePadView` affords grouping all required
+data at once in the `select_from_typepad()` method, which can be retrieved in
+a batch and provided to your view's implementation.
+
+"""
+
+
 from urlparse import urljoin
 from os import path
 import re
@@ -45,10 +59,13 @@ import typepad
 
 
 def parse_etags(etag_str):
-    """
-    Parses a string with one or several etags passed in If-None-Match and
-    If-Match headers by the rules in RFC 2616. Returns a list of etags
-    without surrounding double quotes (") and unescaped from \<CHAR>.
+    """Parses ETags out of an HTTP header value.
+
+    This function parses ETags out of the value of an ``If-None-Match`` or
+    ``If-Match`` header according to the rules in RFC 2616. It returns a list
+    of ETags without surrounding double quotes (``"``) and with no characters
+    escaped with backslashes, ``\<CODE>`` style.
+
     """
     etags = re.findall(r'(?:W/)?"((?:\\.|[^"])*)"', etag_str)
     if not etags:
@@ -59,17 +76,18 @@ def parse_etags(etag_str):
 
 
 class GenericView(http.HttpResponse):
-    """
-    A base view class for TypePadView.
+    """A class-based view.
 
-    This class by default permits 'GET' requests, and will automatically
-    handle conditional requests (respecting If-Modified-Since and Etag HTTP
-    headers).
+    This class provides a generic framework for implementing a class-based
+    view. By default it permits ``GET`` requests, and will automatically
+    handle conditional requests, controlled through ``If-Modified-Since`` and
+    ETag request headers.
+
+    This class works by subclassing ``HttpResponse``. When called, the view is
+    instantiated with the request instance and url parameters, just like a
+    Django view function. The request is handled by the initializer and the
+    result of the instantiation is the response to the given request.
 
-    This class inherits from ``HttpResponse``. When invoked, the view is
-    instantiated, and given the request and url parameters just like a
-    Django view function is. The resulting value is the response that is
-    used for the active request.
     """
     methods = ('GET',)
 
@@ -90,13 +108,14 @@ class GenericView(http.HttpResponse):
 
     def _update(self, response):
         """
-        Merge the info from another response with this instance.
+        Merge another `HttpResponse` into this instance.
+
+        This method copies the attributes from the given response to this
+        instance, with the exceptions of the ``_headers`` and ``cookies``
+        dictionaries. Instead these members' ``update()`` methods are used,
+        preserving headers and cookies present in this response but not the
+        argument.
 
-        This method simply copies the attributes from the given response to
-        this instance, with the exceptions of the ``_headers`` and ``cookies``
-        dictionaries, whose ``update`` methods are called. This means that any
-        headers or cookies which are present in this response but not the
-        argument are preserved.
         """
         self._charset = response._charset
         self._is_string = response._is_string
@@ -134,9 +153,14 @@ class GenericView(http.HttpResponse):
 
     def conditional_dispatch(self, request, *args, **kwargs):
         """
-        For GET, HEAD, and PUT requests, this method calls the ``etag()`` and
-        ``last_modified()`` methods and then checks whether a the appropriate
-        preconditions are satisfied before continuing.
+        Dispatches the request or returns an `HttpResponseNotModified`
+        instance, as appropriate.
+
+        For ``GET``, ``HEAD``, and ``PUT`` requests, this method calls its
+        instance's `etag()` and `last_modified()` methods, then checks
+        whether a the appropriate preconditions are satisfied before
+        continuing.
+
         """
         allowed, response = self._check_request_allowed(request, *args,
                                                         **kwargs)
@@ -191,19 +215,21 @@ class GenericView(http.HttpResponse):
         Returns a value representing the last modification timestamp of the
         view.
 
-        To support If-Modified-Since headers, return a timestamp representing
-        the last-modified date of the view. This may be based on a physical
-        file timestamp, or the last modified element of the view being
-        published.
+        To support ``If-Modified-Since`` headers, return a timestamp
+        representing the last-modified date of the view. This may be based on
+        a physical file timestamp, or the last modified element of the view
+        being published.
+
         """
         return None
 
     def etag(self, request, *args, **kwargs):
         """
-        Returns a value used as the Etag for the view.
+        Returns a value used as the ETag for the view.
+
+        To support ``If-None-Match`` ETag HTTP headers, return an appropriate
+        ETag here.
 
-        To support If-None-Match Etag HTTP headers, return an appropriate
-        etag here.
         """
         return None
 
@@ -211,8 +237,9 @@ class GenericView(http.HttpResponse):
         """
         Handles ``GET`` requests.
 
-        Subclasses should override this method to handle GET requests. The
-        return value should be a HttpResponse instance.
+        Override this method to handle ``GET`` requests, returning an
+        `HttpResponse` instance.
+
         """
         return http.HttpResponse()
 
@@ -224,28 +251,30 @@ class GenericView(http.HttpResponse):
         Subclasses can override this method which is invoked after the class
         has been initialized but prior to the dispatch to the ``GET``,
         ``POST``, etc. handler.
+
         """
         pass
 
-    def render_to_response(self, template, more_context=None):
+    def render_to_response(self, template, more_context=None, **kwargs):
         """
-        A shortcut method that runs the render_to_response
-        Django shortcut.
+        A shortcut method that runs the `render_to_response` Django shortcut.
 
         It will apply the view's context object as the context for rendering
         the template. Additional context variables may be passed in, similar
-        to the render_to_response shortcut.
+        to the `render_to_response` shortcut.
+
         """
         if more_context:
             self.context.push()
             self.context.update(more_context)
-        results = render_to_response(template, self.context)
+        results = render_to_response(template, context_instance=self.context, **kwargs)
         if more_context:
             self.context.pop()
         return results
 
 
 class TypePadView(GenericView):
+
     """
     A subclass of the ``GenericView`` class that adds TypePad-specific
     features and behavior.
@@ -258,54 +287,56 @@ class TypePadView(GenericView):
     batch request is made to the TypePad API for all data elements requested
     in the ``select_from_typepad()`` method.
 
-    View properties:
+    .. rubric:: Class attributes:
 
-    paginate_by: specify this value if the view is to support pagination
-        of a list of objects. An ``object_list`` member is expected to be
-        assigned in order to provide context for pagination. A template
-        context variable named ``page_obj`` is also set, being an instance
-        of ``FinitePaginator``.
-    object_list: Assign to this member when the view is to paginate a
-        list of objects. This member is also assigned to the
-    offset: When paginating a list of objects, this member is
-        automatically assigned, based on the ``page`` parameter to the view
-        and the value of the ``paginate_by`` property. This value may
-        be used to control the selection of rows used to populate the
-        ``object_list`` member during the ``setup()`` or
-        ``select_from_typepad()`` methods.
-    limit: Assigned as the number of rows to select for the ``object_list``
-        member. This is typically set to ``paginate_by``.
-    paginate_template: Assign a string to control the format of
-        next, previous links.
-    form: The Django form class that is to be used for any editable object
-        the view is presenting. If this member is set, it is instantiated
-        during the ``setup()`` method and during a ``POST`` request, the
-        values of the ``POST`` are given to it. Once the form is instantiated
-        it is assigned to a ``form_instance`` member. The form is also
-        assigned to the template context as ``form``.
-    template_name: Assign the name of a Django template file to be
-        used for ``GET`` requests. The value of this property is stripped of
-        any path and suffix and assigned to the 'view' template context
-        variable.
-    login_required: If the view requires an authenticated user to run,
-        set this member to True. It relies on the settings.LOGIN_URL value
-        for redirecting the user to a login form.
+    * ``paginate_by``: specify this value if the view is to support pagination
+      of a list of objects. An ``object_list`` member is expected to be
+      assigned in order to provide context for pagination. A template context
+      variable named ``page_obj`` is also set, being an instance of
+      ``FinitePaginator``.
+    * ``object_list``: Assign to this member when the view is to paginate a
+      list of objects.
+    * ``offset``: When paginating a list of objects, this member is
+      automatically assigned, based on the ``page`` parameter to the view and
+      the value of the ``paginate_by`` property. This value may be used to
+      control the selection of rows used to populate the ``object_list``
+      member during the `setup()` or `select_from_typepad()` methods.
+    * ``limit``: Assigned as the number of rows to select for the
+      ``object_list`` member. This is typically set to ``paginate_by``.
+    * ``paginate_template``: Assign a string to control the format of next,
+      previous links.
+    * ``form``: The Django form class that is to be used for any editable
+      object the view is presenting. If this member is set, it is instantiated
+      during the `setup()` method and during a ``POST`` request, the values of
+      the ``POST`` are given to it. Once the form is instantiated, it is
+      assigned to a ``form_instance`` instance attribute. The form is also
+      assigned to the template context as ``form``.
+    * ``template_name``: Assign the name of a Django template file to be used
+      for ``GET`` requests. The value of this property is stripped of any path
+      and suffix and assigned to the ``view`` template context variable.
+    * ``login_required``: If the view requires an authenticated user to run,
+      set this member to True. It relies on the settings.LOGIN_URL value for
+      redirecting the user to a login form.
 
-    Context variables:
+    .. rubric:: Template variables:
 
-    The following variables are assigned by default and available for any
-    template.
+    These variables are available for any `TypePadView`.
+
+    * ``view``: The basename of the ``template_name`` member, minus any file
+      path and extension.
+    * ``form``: The Django `Form` instance for this view, if this kind of view
+      has a form.
+    * ``page_obj``: The Django `Paginator` instance for this view, if the view
+      is paged.
 
-    view: Assigned as the basename of the ``template_name`` member, minus any
-        file path and extension.
-    form: The value of the ``form_instance`` member.
-    page_obj: Set when ``paginate_by`` member is assigned.
     """
+
     paginate_by = None
     paginate_template = None
     form = None
     template_name = None
     login_required = False
+    admin_required = False
 
     def __init__(self, request, *args, **kwargs):
         self.form_instance = None
@@ -323,13 +354,26 @@ class TypePadView(GenericView):
 
     def select_from_typepad(self, request, *args, **kwargs):
         """
-        Empty method used for issuing TypePad API subrequests for the view.
+        Instantiates the TypePad API resources to display in this view.
+
+        The TypePad API resources instantiated in `select_from_typepad()` are
+        requested *en masse* in one API batch request. Add your resources to
+        the view's `context` dictionary, so they will be available when
+        rendering the template.
+
+        This implementation does nothing, requesting no API resources.
+
         """
         pass
 
-    def filter_object_list(self):
+    def filter_object_list(self, request):
         """
-        Empty method used for any filtering of objects returned by the API.
+        Filters the list of objects returned by the API according to this
+        view's requirements.
+
+        This implementation does nothing, leaving the view's `object_list` as
+        it is.
+
         """
         pass
 
@@ -342,23 +386,29 @@ class TypePadView(GenericView):
                                                             **kwargs)
         if not allowed:
             return allowed, response
-        if self.login_required:
+        if self.login_required or self.admin_required:
             response = login_required(lambda r: False)(request)
             if response:
                 return False, response
+            if self.admin_required:
+                # additionally, user must be an admin
+                if not request.user.is_superuser:
+                    # pretend this url doesn't exist?
+                    raise http.Http404
         return True, None
 
     def typepad_request(self, request, *args, **kwargs):
         """
         Issues a single TypePad batch request to render the current view.
 
-        This method is a wrapping method for the ``select_from_typepad()``
-        method which itself does individual TypePad API subrequests necessary
-        for the view. If a session token is present, the authed user is also
-        fetched with this batch request.
+        You generally needn't override this method; this method wraps the
+        `select_from_typepad()` method, which itself does individual TypePad
+        API subrequests necessary for the view.
+
+        If a session token is present, the authed user is also fetched with
+        this batch request. In addition, the pagination state is set if the
+        ``paginate_by`` attribute is assigned.
 
-        In addition, the pagination state is set if the ``paginate_by``
-        attribute is assigned.
         """
         # Pagination setup
         if self.paginate_by:
@@ -381,8 +431,8 @@ class TypePadView(GenericView):
         typepad.client.complete_batch()
 
         # Page parameter assignment
-        if self.paginate_by and self.object_list:
-            self.filter_object_list()
+        if self.paginate_by and self.object_list is not None:
+            self.filter_object_list(request)
             link_template = self.paginate_template or urljoin(request.path, '/page/%d')
             paginator = FinitePaginator(self.object_list, self.paginate_by,
                                         offset=self.offset,
@@ -393,9 +443,6 @@ class TypePadView(GenericView):
                 raise http.Http404
 
     def setup(self, request, *args, **kwargs):
-        """
-        Initializes a TypePadView instance.
-        """
         super(TypePadView, self).setup(request, *args, **kwargs)
 
         # View parameter from view or template_name
@@ -416,16 +463,17 @@ class TypePadView(GenericView):
 
             self.context['form'] = self.form_instance
 
-        if request.method == 'GET':
-            return self.typepad_request(request, *args, **kwargs)
+        return self.typepad_request(request, *args, **kwargs)
 
     def dispatch(self, request, *args, **kwargs):
         """
-        Dispatch for TypePadView, which handles invalid form POST requests.
+        Dispatches requests to `TypePadView` instances.
+
+        If the dispatched ``POST`` call returns ``None`` for a response and
+        the view's `form_instance` instance member is an invalid form, the
+        view is re-dispatched as a ``GET`` request to display the errors and
+        ask for new valid form input.
 
-        In the event that the form is invalid, the response is unset,
-        the TypePad API requests for the view are issued and the 'get' handler
-        is invoked to return a response.
         """
         response = super(TypePadView, self).dispatch(request, *args, **kwargs)
         if self.form and (request.method == 'POST') and (response is None):
@@ -433,19 +481,19 @@ class TypePadView(GenericView):
             # requests for this view and invoke the GET handler for the
             # response.
             if not self.form_instance.is_valid() or request.flash.get('errors'):
-                response = self.typepad_request(request, *args, **kwargs)
-                if response:
-                    return response
                 response = self.get(request, *args, **kwargs)
         return response
 
     def get(self, request, *args, **kwargs):
         """
-        Base handler for GET requests.
+        Handles a ``GET`` request.
+
+        By default, the template named in the request parameters (or, if none
+        is given, the view's ``template_name`` attribute) is rendered with the
+        view's context (including resources stashed there by the view's
+        `select_from_typepad()` method). If no template name is available,
+        this method returns an empty `HttpResponse` instance.
 
-        When the ``template_name`` attribute is assigned, the template
-        is rendered as the response to the request. Otherwise, an
-        empty HttpResponse is returned.
         """
         template = view = kwargs.get('template_name', self.template_name)
         if template is None:
@@ -454,8 +502,9 @@ class TypePadView(GenericView):
 
 
 class TypePadFeed(Feed):
-    """ A subclass of the Django ``Feed`` class that handles selecting data
-    from the TypePad client library. """
+
+    """A subclass of the Django `Feed` class that handles selecting data from
+    the TypePad client library."""
 
     feed_type = Atom1Feed
 
@@ -470,8 +519,8 @@ class TypePadFeed(Feed):
 
 
 class TypePadEventFeed(TypePadFeed):
-    """ A subclass of the ``TypePadFeed`` class that handles serving
-    Asset items. """
+
+    """A subclass of `TypePadFeed` that handles serving `Asset` items."""
 
     def item_link(self, event):
         return event.object.get_absolute_url()
